@@ -14,19 +14,51 @@
 #ifndef GENSIO_H
 #define GENSIO_H
 
+#include <stdarg.h>
+#include <stdint.h>
+#include <stdbool.h>
+#include <stdio.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 #include <gensio/gensio_dllvisibility.h>
+#include <gensio/gensio_types.h>
 #include <gensio/gensio_deprecated.h>
-#include <gensio/gensio_os_funcs.h>
 #include <gensio/gensio_err.h>
 #include <gensio/gensio_version.h>
 
-struct gensio;
+/*
+ * This should eventually go away in preference to the functions given
+ * inthe include just below.
+ */
+#include <gensio/gensio_os_funcs.h>
 
-typedef unsigned long gensiods; /* Data size */
+/*
+ * The gensio_os_funcs_xxx() functions in the following are accessors
+ * for user-visible operations on the os functions.  Do not use the
+ * functions in gensio_os_funcs directly from user code.
+ */
+#include <gensio/gensio_os_funcs_public.h>
+
+/*
+ * A bitmask of log levels to tell what to log.  Defaults to fatal and err
+ * only.
+ */
+GENSIO_DLL_PUBLIC
+void gensio_set_log_mask(unsigned int mask);
+GENSIO_DLL_PUBLIC
+unsigned int gensio_get_log_mask(void);
+GENSIO_DLL_PUBLIC
+const char *gensio_log_level_to_str(enum gensio_log_levels level);
+
+GENSIO_DLL_PUBLIC
+void gensio_vlog(struct gensio_os_funcs *o, enum gensio_log_levels level,
+		 const char *str, va_list args);
+GENSIO_DLL_PUBLIC
+void gensio_log(struct gensio_os_funcs *o, enum gensio_log_levels level,
+		const char *str, ...);
 
 /*
  * The following are documented in gensio_event.3
@@ -56,21 +88,6 @@ typedef unsigned long gensiods; /* Data size */
 #define GENSIO_EVENT_USER_MIN		100000
 #define GENSIO_EVENT_USER_MAX		199999
 
-typedef int (*gensio_event)(struct gensio *io, void *user_data,
-			    int event, int err,
-			    unsigned char *buf, gensiods *buflen,
-			    const char *const *auxdata);
-
-/*
- * Callbacks for functions that don't give an error (close);
- */
-typedef void (*gensio_done)(struct gensio *io, void *open_data);
-
-/*
- * Callbacks for functions that give an error (open);
- */
-typedef void (*gensio_done_err)(struct gensio *io, int err, void *open_data);
-
 GENSIO_DLL_PUBLIC
 int str_to_gensio(const char *str,
 		  struct gensio_os_funcs *o,
@@ -97,12 +114,6 @@ int gensio_write(struct gensio *io, gensiods *count,
 		 const void *buf, gensiods buflen,
 		 const char *const *auxdata);
 
-/* Purposefully exactly the same as iovev (see writev(2)) */
-struct gensio_sg {
-    const void *buf;
-    gensiods buflen;
-};
-
 GENSIO_DLL_PUBLIC
 int gensio_write_sg(struct gensio *io, gensiods *count,
 		    const struct gensio_sg *sg, gensiods sglen,
@@ -165,39 +176,8 @@ void gensio_set_write_callback_enable(struct gensio *io, bool enabled);
 GENSIO_DLL_PUBLIC
 int gensio_control(struct gensio *io, int depth, bool get,
 		   unsigned int option, char *data, gensiods *datalen);
-#define GENSIO_CONTROL_DEPTH_ALL	-1
-#define GENSIO_CONTROL_DEPTH_FIRST	-2
 
-#define GENSIO_CONTROL_GET	true
-#define GENSIO_CONTROL_SET	false
-
-#define GENSIO_CONTROL_NODELAY			1
-#define GENSIO_CONTROL_STREAMS			2
-#define GENSIO_CONTROL_SEND_BREAK		3
-#define GENSIO_CONTROL_GET_PEER_CERT_NAME	4
-#define GENSIO_CONTROL_CERT_AUTH		5
-#define GENSIO_CONTROL_USERNAME			6
-#define GENSIO_CONTROL_SERVICE			7
-#define GENSIO_CONTROL_CERT			8
-#define GENSIO_CONTROL_CERT_FINGERPRINT		9
-#define GENSIO_CONTROL_ENVIRONMENT		10
-#define GENSIO_CONTROL_MAX_WRITE_PACKET		11
-#define GENSIO_CONTROL_ARGS			12
-#define GENSIO_CONTROL_EXIT_CODE		13
-#define GENSIO_CONTROL_WAIT_TASK		14
-#define GENSIO_CONTROL_ADD_MCAST		15
-#define GENSIO_CONTROL_DEL_MCAST		16
-#define GENSIO_CONTROL_LADDR			17
-#define GENSIO_CONTROL_LPORT			18
-#define GENSIO_CONTROL_CLOSE_OUTPUT		19
-#define GENSIO_CONTROL_CONNECT_ADDR_STR		20
-#define GENSIO_CONTROL_RADDR			21
-#define GENSIO_CONTROL_RADDR_BIN		22
-#define GENSIO_CONTROL_REMOTE_ID		23
-#define GENSIO_CONTROL_PASSWORD			27
-#define GENSIO_CONTROL_2FA			28
-#define GENSIO_CONTROL_AUX_DATA			29
-#define GENSIO_CONTROL_REM_AUX_DATA		30
+#include <gensio/gensio_control.h>
 
 GENSIO_DLL_PUBLIC
 const char *gensio_get_type(struct gensio *io, unsigned int depth);
@@ -237,9 +217,35 @@ int gensio_write_s_intr(struct gensio *io, gensiods *count,
 			const void *data, gensiods datalen,
 			gensio_time *timeout);
 
+/*
+ * Increment the gensio's refcount.  Internally there are situations
+ * where one piece of code passes a gensio into another piece of code,
+ * and that other piece of code that might free it on an error, but
+ * the upper layer gets the error and wants to free it, too.  This
+ * keeps it around for that situation.  It may also be used externally
+ * for refcounting, every ref will require an extra free.
+ */
+GENSIO_DLL_PUBLIC
+void gensio_ref(struct gensio *io);
 
-struct gensio_accepter;
+/*
+ * This is data for frameworks that sit on top of gensio, added to do
+ * a clean addition of a C++ framework on top of gensio.  It's data
+ * you can set on a gensio that has a destructor that is called when
+ * the final free is done on the gensio data.
+ */
+struct gensio_frdata {
+    void (*freed)(struct gensio *io, struct gensio_frdata *frdata);
+};
 
+GENSIO_DLL_PUBLIC
+void gensio_set_frdata(struct gensio *io, struct gensio_frdata *frdata);
+GENSIO_DLL_PUBLIC
+struct gensio_frdata *gensio_get_frdata(struct gensio *io);
+
+/****** Accepters ******/
+
+GENSIO_DLL_PUBLIC
 struct gensio_accepter *gensio_acc_get_child(struct gensio_accepter *acc,
 					     unsigned int depth);
 
@@ -273,14 +279,6 @@ struct gensio_acc_postcert_verify_data {
 #define GENSIO_ACC_EVENT_REQUEST_2FA		9
 /* Uses struct gensio_acc_password_verify_data */
 
-typedef int (*gensio_accepter_event)(struct gensio_accepter *accepter,
-				     void *user_data, int event, void *data);
-
-/*
- * Callbacks for functions that don't give an error (shutdown);
- */
-typedef void (*gensio_acc_done)(struct gensio_accepter *acc, void *cb_data);
-
 GENSIO_DLL_PUBLIC
 int str_to_gensio_accepter(const char *str, struct gensio_os_funcs *o,
 			   gensio_accepter_event cb, void *user_data,
@@ -337,14 +335,6 @@ int gensio_acc_set_accept_callback_enable_s(struct gensio_accepter *accepter,
 GENSIO_DLL_PUBLIC
 int gensio_acc_control(struct gensio_accepter *accepter, int depth, bool get,
 		       unsigned int option, char *data, gensiods *datalen);
-/* Uses GENSIO_CONTROL_DEPTH_xxx options above. */
-#define GENSIO_ACC_CONTROL_LADDR	1
-#define GENSIO_ACC_CONTROL_LPORT	2
-
-/*
- * Get/set the tcpdname option for the gensio dynamically.
- */
-#define GENSIO_ACC_CONTROL_TCPDNAME	3
 
 GENSIO_DLL_PUBLIC
 int gensio_acc_set_sync(struct gensio_accepter *acc);
@@ -372,7 +362,7 @@ bool gensio_acc_exit_on_close(struct gensio_accepter *accepter);
 GENSIO_DLL_PUBLIC
 const char *gensio_acc_get_type(struct gensio_accepter *acc,
 				unsigned int depth);
-    
+
 GENSIO_DLL_PUBLIC
 bool gensio_acc_is_reliable(struct gensio_accepter *accepter);
 GENSIO_DLL_PUBLIC
@@ -380,32 +370,33 @@ bool gensio_acc_is_packet(struct gensio_accepter *accepter);
 GENSIO_DLL_PUBLIC
 bool gensio_acc_is_message(struct gensio_accepter *accepter);
 
+/*
+ * This is data for frameworks that sit on top of gensio, added to do
+ * a clean addition of a C++ framework on top of gensio.  It's data
+ * you can set on a gensio that has a destructor that is called when
+ * the final free is done on the gensio data.
+ */
+struct gensio_acc_frdata {
+    void (*freed)(struct gensio_accepter *acc,
+		  struct gensio_acc_frdata *frdata);
+};
+
+GENSIO_DLL_PUBLIC
+void gensio_acc_set_frdata(struct gensio_accepter *acc,
+			   struct gensio_acc_frdata *frdata);
+GENSIO_DLL_PUBLIC
+struct gensio_acc_frdata *gensio_acc_get_frdata(struct gensio_accepter *acc);
+
 /*
  * These are the low-level network protocol that gensio support.  Used
- * mostly in interacting with addresses.  zero is reserved.
+ * mostly in interacting with addresses, anything named protocol.
+ * zero is reserved.
  */
 #define GENSIO_NET_PROTOCOL_TCP 1
 #define GENSIO_NET_PROTOCOL_UDP 2
 #define GENSIO_NET_PROTOCOL_SCTP 3
 #define GENSIO_NET_PROTOCOL_UNIX 4
 
-/*
- * Gensio address structure
- *
- * This is used to hide the details of address handling for network
- * gensios.  A gensio_addr has a set of addresses embedded in it.  The
- * list is immutable after allocation.
- *
- * The address has the concept of a current address in it that can be
- * iterated.  You get an address, and you can use the iterator
- * function to iterate over it and extract information from the
- * individual addresses.
- *
- * Note that some function use the current address, and some use all
- * the addresses.
- */
-struct gensio_addr;
-
 /*
  * Dealing with iterators.
  */
@@ -414,8 +405,6 @@ void gensio_addr_rewind(struct gensio_addr *addr);
 /* Return false if no more addresses exist. */
 GENSIO_DLL_PUBLIC
 bool gensio_addr_next(struct gensio_addr *addr);
-GENSIO_DLL_PUBLIC
-bool gensio_addr_at_end(struct gensio_addr *addr);
 /*
  * Gets the current address.  len must be provided, it is the size of
  * the buffer and is updated to the actual size (which may be larger
@@ -429,6 +418,7 @@ void gensio_addr_getaddr(const struct gensio_addr *addr,
 #define GENSIO_NETTYPE_IPV4	1
 #define GENSIO_NETTYPE_IPV6	2
 #define GENSIO_NETTYPE_UNIX	3
+#define GENSIO_NETTYPE_AX25	4
 
 /*
  * Create a gensio address from raw address data.  Note that the iaddr
@@ -510,17 +500,6 @@ struct gensio_addr *gensio_addr_cat(const struct gensio_addr *ai1,
 GENSIO_DLL_PUBLIC
 void gensio_addr_free(struct gensio_addr *ai);
 
-/*
- * Compare two addresses.  If compare_ports is false, ignore ports in
- * the comparison, otherwise compare the ports.  If all_addr is true,
- * compare all the sub-addresses in the addresses.  Otherwise only
- * compare the current address.
- */
-GENSIO_DLL_PUBLIC
-bool gensio_addr_cmp(const struct gensio_addr *addr1,
-		     const struct gensio_addr *addr2,
-		     bool compare_ports, bool all_addr);
-
 /*
  * See if addr is present in ai.  Ports are not compared unless
  * compare_ports is true.
@@ -617,25 +596,26 @@ int gensio_reset_defaults(struct gensio_os_funcs *o);
 
 GENSIO_DLL_PUBLIC
 int gensio_set_default(struct gensio_os_funcs *o,
-		       const char *class, const char *name,
+		       const char *classstr, const char *name,
 		       const char *strval, int intval);
 
 
 GENSIO_DLL_PUBLIC
 int gensio_get_default(struct gensio_os_funcs *o,
-		       const char *class, const char *name, bool classonly,
+		       const char *classstr, const char *name, bool classonly,
 		       enum gensio_default_type type,
 		       char **strval, int *intval);
 
 GENSIO_DLL_PUBLIC
 int gensio_get_defaultaddr(struct gensio_os_funcs *o,
-			   const char *class, const char *name, bool classonly,
+			   const char *classstr, const char *name,
+			   bool classonly,
 			   int iprotocol, bool listen, bool require_port,
 			   struct gensio_addr **rai);
 
 GENSIO_DLL_PUBLIC
 int gensio_del_default(struct gensio_os_funcs *o,
-		       const char *class, const char *name, bool delclasses);
+		       const char *classstr, const char *name, bool delclasses);
 
 /*
  * Clean up all the internal gensio memory.  Not really necessary, but
@@ -757,13 +737,21 @@ char *gensio_alloc_vsprintf(struct gensio_os_funcs *o,
 GENSIO_DLL_PUBLIC
 char *gensio_alloc_sprintf(struct gensio_os_funcs *o,
 			   const char *fmt, ...);
-    
+
 GENSIO_DLL_PUBLIC
 char *gensio_strdup(struct gensio_os_funcs *o, const char *str);
 
-#ifdef __cplusplus
-}
-#endif
+GENSIO_DLL_PUBLIC
+char *gensio_strndup(struct gensio_os_funcs *o, const char *str, gensiods len);
+
+/*
+ * Take the input string, put " around it, and put a \ infront of
+ * every \ and ".  This allows you to take a string with " and \ in it
+ * and have them pass through the str_to_gensio() and such functions
+ * properly.
+ */
+GENSIO_DLL_PUBLIC
+char *gensio_quote_string(struct gensio_os_funcs *o, const char *str);
 
 /*
  * Return the number of allocated gensios.  This is primarily for
@@ -772,4 +760,43 @@ char *gensio_strdup(struct gensio_os_funcs *o, const char *str);
 GENSIO_DLL_PUBLIC
 gensiods gensio_num_alloced(void);
 
+/* For testing, do not use in normal code. */
+GENSIO_DLL_PUBLIC
+void gensio_osfunc_exit(int rv);
+
+/*
+ * Generic functions for dumping buffer data to stdio
+ */
+struct gensio_fdump {
+    unsigned int column;
+    unsigned int pos;
+    unsigned int indent;
+    unsigned char data[16];
+};
+
+/*
+ * Call this before using an fdump structure.
+ */
+GENSIO_DLL_PUBLIC
+void gensio_fdump_init(struct gensio_fdump *h, unsigned int indent);
+
+/*
+ * Format len bytes of data in buf to file f.  You can call this
+ * multiple times and it will continue the dump where it left off.
+ */
+GENSIO_DLL_PUBLIC
+void gensio_fdump_buf(FILE *f, const unsigned char *buf, gensiods len,
+		      struct gensio_fdump *h);
+
+/*
+ * When you are done dumping a buffer, call this.  It will output the
+ * last bits of information.
+ */
+GENSIO_DLL_PUBLIC
+void gensio_fdump_buf_finish(FILE *f, struct gensio_fdump *h);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* GENSIO_H */
